{******************************************************************}
{*                                                                *}
{*                         Base Calculator                        *}
{*                                                                *}
{*                     (C) J.N.Zaitseff, 1993.                    *}
{*                                                                *}
{******************************************************************}

{ This calculator allows you to calculate in decimal, hexadecimal,
  binary and octal. The functions supported are + - * / NEG NOT AND
  OR XOR EQV. }

program BaseCalc;

{ $DEFINE Debug}

{$IFNDEF WINDOWS} Error: Windows compilation only. {$ENDIF}

{$A+,B-,F-,G-,I+,N-,V+,W+,X+}
{$C Moveable, DemandLoad, Discardable}
{$D Base calculator.}
{$R BaseCalc}

{$IFDEF Debug} {$D+,L+,R+,S+} {$ELSE} {$D-,R-,S-} {$ENDIF}

uses
  OWindows, ODialogs, WinTypes, WinProcs, Strings, BCalcDef;

const
  BaseCalcHelpFile = '';

  AppName : pchar = 'BaseCalc';

  RGB_Yellow = $0000FFFF;
  RGB_Blue   = $00FF0000;

type

  Calc_Mode  = (Decimal,Hexadecimal,Binary,Octal);
  Calc_Size  = (ByteSize,WordSize,DWordSize);
  Calc_State = (cs_First, cs_Valid, cs_Error);

  PValueDlg = ^TValueDlg;
  TValueDlg = object(TDialog)
                Value : longint;
                Size  : Calc_Size; 
                constructor Init (AParent : PWindowsObject; AResource : pchar;
                                  AValue : longint; ASize : Calc_Size);
                procedure SetupWindow; virtual;
              end;  
                
  PBaseCalc = ^TBaseCalc;
  TBaseCalc = object(TDlgWindow)
                BlueBrush  : HBrush;
                X, Y       : integer;       

                CalcMode   : Calc_Mode;
                CalcSize   : Calc_Size;
                CalcState  : Calc_State;

                DisplayStr      : string[63];
                CalcMem         : longint;
                CalcPrev        : longint;
                CalcPrevUsed    : boolean;
                CalcOp          : word;
                CalcFirstEquals : boolean;

                MaxVal,
                MaxLen     : word;
                MaxSize    : longint;

                constructor Init;
                procedure   SetupWindow; virtual;
                destructor  Done; virtual;
                function    GetClassName : pchar; virtual;
                procedure   GetWindowClass (var AWndClass : TWndClass); virtual;
                procedure   UpdateDisplay;

                procedure WMControlColor (var Msg : TMessage); virtual wm_First + wm_CtlColor;
                procedure WMPaint (var Msg : TMessage); virtual wm_First + wm_Paint;
                procedure WMCommand (var Msg : TMessage); virtual wm_First + wm_Command; 

                procedure UpdateButtons;
                procedure FlashButton (ButtonID : integer);
                procedure SetBaseBtn (ButtonID : integer);
                procedure SetSizeBtn (ButtonID : integer);

                procedure NotImplemented;

                procedure CMSaveConfig (var Msg : TMessage); virtual cm_First + cm_SaveConfig;
                procedure CMQuit (var Msg : TMessage); virtual cm_First + cm_Quit;

                procedure CMPaste (var Msg : TMessage); virtual cm_First + cm_EditPaste;
                procedure CMCopy (var Msg : TMessage); virtual cm_First + cm_EditCopy;

                procedure CMValue (var Msg : TMessage); virtual cm_First + cm_Value;

                procedure CMHelpIndex (var Msg : TMessage); virtual cm_First + cm_HelpIndex;
                procedure CMHelpKeyboard (var Msg : TMessage); virtual cm_First + cm_HelpKeyboard;
                procedure CMHelpCommands (var Msg : TMessage); virtual cm_First + cm_HelpCommands;
                procedure CMHelpProcedures (var Msg : TMessage); virtual cm_First + cm_HelpProcedures;
                procedure CMUsingHelp (var Msg : TMessage); virtual cm_First + cm_UsingHelp;
                procedure CMAbout (var Msg : TMessage); virtual cm_First + cm_About;

                procedure InitCalculator;
                procedure Calc_SetInitValues;
                procedure Calc_Clear;
                procedure Calc_NumKey (Value : word);
                procedure Calc_FncKey (Key : word);
                procedure Calc_SaveSettings;
              end;

  PBaseCalcApp = ^TBaseCalcApp;
  TBaseCalcApp = object(TApplication)
                   procedure InitMainWindow; virtual;
                   procedure InitInstance; virtual;
                   function ProcessAppMsg (var Message : TMsg) : boolean; virtual;
                 end;


function ValueOf (Mode : Calc_Mode; St : string) : longint;

var
  Mult,
  I, J : byte;
  Acc  : longint;

begin
  case Mode of
    Decimal     : Mult := 10;
    Hexadecimal : Mult := 16;
    Binary      : Mult := 2;
    Octal       : Mult := 8
  end;
  Acc := 0;
  for I := 1 to length(St) do
    begin
      J := ord(St[I]) - ord('0');
      if J > 9 then
        J := J - 7;
      Acc := Acc * Mult + J
    end;
  ValueOf := Acc 
end;

function StringOf (Mode : Calc_Mode; Value : longint) : string;

const
  Table : array [0..15] of char = '0123456789ABCDEF';

var
  Acc  : string[33];
  Mult,
  I, J : byte;
  R    : real;

begin
  case Mode of
    Decimal     : Mult := 10;
    Hexadecimal : Mult := 16;
    Binary      : Mult := 2;
    Octal       : Mult := 8
  end;
  Acc := '';
  if Value < 0 then
    R := Value + 4294967296.0;
  repeat
    if Value < 0 then
      begin
        I := round(frac(R / Mult) * Mult);
        R := int(R / Mult);
        if R <= MaxLongInt then
          Value := round(R)
      end
    else
      begin  
        I := Value mod Mult;
        Value := Value div Mult
      end;
    Acc := Table[I] + Acc
  until Value = 0;
  StringOf := Acc      
end; 

function HigherInPrecedence (Op1, Op2 : word) : boolean;

function OpPrecedence (Op : word) : word;

begin
  case Op of
    BCalc_NotID,
    BCalc_NegID    : OpPrecedence := 3;
    BCalc_TimesID,
    BCalc_DivID,
    BCalc_ModID,
    BCalc_AndID    : OpPrecedence := 2;
    BCalc_PlusID,
    BCalc_MinusID,
    BCalc_OrID,
    BCalc_XorID,
    BCalc_EqvID    : OpPrecedence := 1
  end
end;

begin
  HigherInPrecedence := OpPrecedence(Op1) >= OpPrecedence(Op2)
end;

procedure RoundVal (Size : Calc_Size; var Value : longint);

begin
  case Size of
    ByteSize : Value := Value and $000000FF;
    WordSize : Value := Value and $0000FFFF
  end
end;


{ Display value dialog methods }

constructor TValueDlg.Init;

begin
  TDialog.Init(AParent,AResource);
  Value := AValue;
  Size := ASize
end;

procedure TValueDlg.SetupWindow;

var
  St  : string[33];
  I   : word;

begin
  TDialog.SetupWindow;
  case Size of
    ByteSize  : str(shortint(Value),St);
    WordSize  : str(integer(Value),St);
    DWordSize : str(longint(Value),St)
  end;
  St := St + #0;
  SetWindowText(GetDlgItem(HWindow,BCalc_SDecValID),@St[1]);
  St := StringOf(Decimal,Value) + #0;
  SetWindowText(GetDlgItem(HWindow,BCalc_DecValID),@St[1]);
  St := StringOf(Hexadecimal,Value) + #0;
  SetWindowText(GetDlgItem(HWindow,BCalc_HexValID),@St[1]);
  St := StringOf(Binary,Value) + #0;
  SetWindowText(GetDlgItem(HWindow,BCalc_BinValID),@St[1]);
  St := StringOf(Octal,Value) + #0;
  SetWindowText(GetDlgItem(HWindow,BCalc_OctValID),@St[1]);
  case Size of
    ByteSize  : I := str_ByteSizeID; 
    WordSize  : I := str_WordSizeID;
    DWordSize : I := str_DWordSizeID;
  end;
  LoadString(HInstance,I,@St,sizeof(St));
  SetWindowText(GetDlgItem(HWindow,BCalc_ModeValID),@St)
end;


{ Base calculator methods }

constructor TBaseCalc.Init;

type
  pstring = array [0..31] of char;

var
  St     : array [1..3] of pstring;
  pSt    : pchar;
  St1    : string[127];
  I, J   : word;
  DoSave,
  IsOK   : boolean;

begin
  TDlgWindow.Init(nil,MakeIntResource(BCalcDlg));
  DoSave := false;
  if GetProfileString(AppName,nil,nil,@St[1],sizeof(St[1])) = 0 then
    begin
      DoSave := true;
      X := -1;
      Y := -1;
      CalcMode := Decimal;
      CalcSize := DWordSize
    end
  else
    begin
      X := -1;
      Y := -1;
      LoadString(HInstance,str_OriginID,St[1],sizeof(St[1]));
      LoadString(HInstance,str_DefOriginID,St[2],sizeof(St[2]));
      GetProfileString(AppName,St[1],St[2],St[3],sizeof(St[3]));
      if St[3][0] = '(' then
        begin
          pSt := StrPos(@St[3][1],',');
          if pSt <> nil then
            begin
              I := pSt - @St[3][1];
              St[3][I+1] := #0;
              St1 := StrPas(@St[3][1]);
              val(St1,X,I);
              inc(pSt);
              St1 := StrPas(pSt);
              dec(St1[0]);
              val(St1,Y,J);
              if (I <> 0) or (J <> 0) then
                begin
                  X := -1;
                  Y := -1;
                  DoSave := true
                end
            end
          else
            DoSave := true
        end
      else
        DoSave := true; 
      LoadString(HInstance,str_ModeID,St[1],sizeof(St[1]));
      GetProfileString(AppName,St[1],nil,St[2],sizeof(St[2]));
      CalcMode := Decimal;
      IsOK := false;
      for I := str_DecID to str_OctID do
        begin
          LoadString(HInstance,I,St[1],sizeof(St[1]));
          if StrIComp(St[1],St[2]) = 0 then
            begin
              CalcMode := Calc_Mode(I - str_DecID);
              IsOK := true
            end
        end;
      if not IsOK then
        DoSave := true;
      LoadString(HInstance,str_SizeID,St[1],sizeof(St[1]));
      GetProfileString(AppName,St[1],nil,St[2],sizeof(St[2]));
      CalcSize := DWordSize;
      IsOK := false;
      for I := str_BSizeID to str_DWSizeID do
        begin
          LoadString(HInstance,I,St[1],sizeof(St[1]));
          if StrIComp(St[1],St[2]) = 0 then
            begin
              CalcSize := Calc_Size(I - str_BSizeID);
              IsOK := true
            end
        end;
      if not IsOK then
        DoSave := true
    end;
  BlueBrush := CreateSolidBrush(RGB_Blue);
  if DoSave then
    begin
      LoadString(HInstance,str_ModifyINIfileID,@St1,sizeof(St1));
      LoadString(HInstance,str_ModifyHeadingID,@St[1],sizeof(St[1]));
      if MessageBox(HWindow,@St1,@St[1],mb_YesNo+mb_IconQuestion) = id_Yes then
        Calc_SaveSettings
    end
end;

procedure TBaseCalc.SetupWindow;

var
  ButtonID : word;

begin
  if (X <> -1) or (Y <> -1) then  
    SetWindowPos(HWindow,0,X,Y,0,0,swp_NoSize);
  TDlgWindow.SetupWindow;
  InitCalculator;
  case CalcMode of
    Decimal     : ButtonID := BCalc_DecBtnID;
    Hexadecimal : ButtonID := BCalc_HexBtnID;
    Binary      : ButtonID := BCalc_BinBtnID;
    Octal       : ButtonID := BCalc_OctBtnID
  end;
  SetBaseBtn(ButtonID);
  case CalcSize of
    ByteSize  : ButtonID := BCalc_ByteBtnID;
    WordSize  : ButtonID := BCalc_WordBtnID;
    DWordSize : ButtonID := BCalc_DWordBtnID
  end;
  SetSizeBtn(ButtonID);
  UpdateDisplay
end;

procedure TBaseCalc.UpdateDisplay;

var
  St : string;

begin
  St := DisplayStr + ' '#0;
  SetWindowText(GetDlgItem(HWindow,BCalc_ResultTextID),@St[1]);
  if CalcMem = 0 then
    St := #0
  else
    St := 'M'#0;
  SetWindowText(GetDlgItem(HWindow,BCalc_MTextID),@St[1])
end;

destructor TBaseCalc.Done;

begin
  DeleteObject(BlueBrush);
  TDlgWindow.Done
end;

function TBaseCalc.GetClassName;

begin
  GetClassName := AppName
end;

procedure TBaseCalc.GetWindowClass;

begin
  TDlgWindow.GetWindowClass(AWndClass);
  AWndClass.hIcon := LoadIcon(HInstance,MakeIntResource(BCalcIcon))
end;

procedure TBaseCalc.WMControlColor;

begin
  case Msg.LParamHi of
    ctlColor_Btn    : Msg.Result := GetStockObject(null_Brush);
    ctlColor_Static : begin
                        SetTextColor(Msg.WParam, RGB_Yellow);
                        SetBkMode(Msg.WParam, transparent);
                        Msg.Result := GetStockObject(black_Brush)
                      end;
    ctlcolor_Dlg    : begin
                        SetBkMode(Msg.WParam, Transparent);
                        Msg.Result := BlueBrush
                      end
  else  
    DefWndProc(Msg)
  end
end;

procedure TBaseCalc.WMPaint;

procedure RectToPoly (X1, Y1, X2, Y2 : integer; var Poly);

var
  Polygon : array [1..4] of TPoint absolute Poly;

begin
  with Polygon[1] do
    begin
      X := X1;
      Y := Y1
    end;
  with Polygon[2] do
    begin
      X := X2;
      Y := Y1
    end;
  with Polygon[3] do
    begin
      X := X2;
      Y := Y2
    end;
  with Polygon[4] do
    begin
      X := X1;
      Y := Y2
    end
end;

var
  PS       : TPaintStruct;
  OldBrush : HBrush;
  OldPen   : HPen;
  R        : TRect;
  Box      : array [1..4] of TPoint;

begin
  BeginPaint(HWindow,PS);
  OldBrush := SelectObject(PS.HDC, GetStockObject(White_Brush));
  OldPen := SelectObject(PS.HDC, GetStockObject(Null_Pen));
  GetClientRect(HWindow, R);
  with R do
    RectToPoly(Left+5,Top+37,Right-7,Top+84,Box);
  Polygon(PS.HDC,Box,4);
  with R do
    RectToPoly(Left+27,Top+8,Right-19,Top+28,Box);
  Polygon(PS.HDC,Box,4);
  SelectObject(PS.HDC, OldPen);
  SelectObject(PS.HDC, OldBrush);
  EndPaint(HWindow,PS)
end;


procedure TBaseCalc.FlashButton;

var
  Button : HWnd;
  Delay  : word;

begin
  if ((ButtonID < BCalc_0ID) or (ButtonID > BCalc_FID)) and
     ((ButtonID < BCalc_EqualsID) or (ButtonID > BCalc_EqvID)) then
    exit;
  Button := GetDlgItem(HWindow,ButtonID);
  SendMessage(Button,bm_SetState,1,0);
  for Delay := 1 to 50000 do;
  SendMessage(Button,bm_SetState,0,0)
end;

procedure TBaseCalc.UpdateButtons;

var
  R : TRect;

begin
  GetClientRect(HWindow,R);
  with R do
    begin
      Left := Left + 10;                      
      Top := Top + 41;
      Right := Right - {37} 20;
      Bottom := Top + {13} 50
    end;
  InvalidateRect(HWindow,@R,false);
end;

procedure TBaseCalc.SetBaseBtn;

var
  R        : TRect;
  OrigMode : Calc_Mode;
  BtnAttr  : longint;

begin
  if CalcState = cs_Error then
    begin
      MessageBeep(mb_OK);
      UpdateButtons;
      exit
    end;
  OrigMode := CalcMode;
  case ButtonID of
    BCalc_DecBtnID : CalcMode := Decimal;
    BCalc_HexBtnID : CalcMode := Hexadecimal;
    BCalc_BinBtnID : if CalcSize = DWordSize then
                       begin
                         MessageBeep(mb_OK);
                         exit
                       end
                     else
                       CalcMode := Binary;
    BCalc_OctBtnID : CalcMode := Octal
  end;
  BtnAttr := ws_Child or ws_Visible or bs_RadioButton;
  if CalcMode = Binary then
    BtnAttr := ws_Disabled or BtnAttr;
  SetWindowLong(GetDlgItem(HWindow,BCalc_DWordBtnID),gwl_Style,BtnAttr);
  CheckRadioButton(HWindow,BCalc_DecBtnID,BCalc_OctBtnID,ButtonID);
  DisplayStr := StringOf(CalcMode,ValueOf(OrigMode,DisplayStr));
  Calc_SetInitValues;
  UpdateButtons;
  UpdateDisplay
end;

procedure TBaseCalc.SetSizeBtn;

var
  R        : TRect;
  OrigSize : Calc_Size;
  Value    : longint;
  BtnAttr  : longint;

begin
  if CalcState = cs_Error then
    begin
      MessageBeep(mb_OK);
      exit
    end;
  OrigSize := CalcSize;
  case ButtonID of
    BCalc_ByteBtnID  : CalcSize := ByteSize;
    BCalc_WordBtnID  : CalcSize := WordSize;
    BCalc_DWordBtnID : if CalcMode = Binary then
                         begin
                           MessageBeep(mb_OK);
                           exit
                         end
                       else  
                         CalcSize := DWordSize
  end;
  BtnAttr := ws_Child or ws_Visible or bs_RadioButton;
  if CalcSize = DWordSize then
    BtnAttr := BtnAttr or ws_Disabled;
  SetWindowLong(GetDlgItem(HWindow,BCalc_BinBtnID),gwl_Style,BtnAttr);
  CheckRadioButton(HWindow,BCalc_ByteBtnID,BCalc_DWordBtnID,ButtonID);
  Value := ValueOf(CalcMode,DisplayStr);
  RoundVal(CalcSize,Value);
  DisplayStr := StringOf(CalcMode,Value);
  RoundVal(CalcSize,CalcMem);
  Calc_SetInitValues;
  UpdateButtons;
  UpdateDisplay
end;

procedure TBaseCalc.WMCommand;

begin
  if Msg.LParamHi = 1 then  { Key press? }
    FlashButton(Msg.WParam);
  case Msg.WParam of
    BCalc_0ID..BCalc_FID              : Calc_NumKey(Msg.WParam - BCalc_0ID);
    BCalc_EqualsID..BCalc_EqvID       : Calc_FncKey(Msg.WParam);
    BCalc_DecBtnID..BCalc_OctBtnID    : SetBaseBtn(Msg.WParam);
    BCalc_ByteBtnID..BCalc_DWordBtnID : SetSizeBtn(Msg.WParam);
  else
    TDlgWindow.WMCommand(Msg)
  end 
end; 

procedure TBaseCalc.CMSaveConfig;

var
  R : TRect;

begin
  GetWindowRect(HWindow,R);
  with R do
    begin
      X := Left;
      Y := Top
    end;
  Calc_SaveSettings
end;

procedure TBaseCalc.CMQuit;

begin
  WinHelp(HWindow,BaseCalcHelpFile,Help_Quit,0);
  TDlgWindow.CMExit(Msg)
end;

procedure TBaseCalc.CMCopy;

var
  StHdl : THandle;
  St    : pchar;

begin
  if not OpenClipboard(HWindow) then
    MessageBeep(mb_IconStop)
  else
    begin
      EmptyClipboard;
      StHdl := GlobalAlloc(Ghnd or GMem_NoDiscard,length(DisplayStr)+1);
      St := GlobalLock(StHdl);
      if St <> nil then
        begin
          move(DisplayStr[1],St[0],length(DisplayStr));
          GlobalUnlock(StHdl);
          SetClipboardData(cf_Text,StHdl)
        end;
      CloseClipboard 
    end
end;

procedure TBaseCalc.CMPaste;

var
  St    : pchar;
  StHdl : THandle;
  C     : char;
  Value : word;

begin
  if not OpenClipboard(HWindow) then
    MessageBeep(mb_IconStop)
  else
    begin
      StHdl := GetClipboardData(cf_Text);
      St := GlobalLock(StHdl);
      if St <> nil then
        begin
          while St[0] <> #0 do
            begin
              C := St[0];
              inc(St);
              C := upcase(C);
              if C in ['0'..'9','A'..'F'] then
                begin
                  Value := ord(C) - ord('0');
                  if Value > 9 then
                    Value := Value - 7;
                  Calc_NumKey(Value)
                end
              else
                MessageBeep(mb_OK)
            end;
          GlobalUnlock(StHdl)
        end;
      CloseClipboard
    end
end;

procedure TBaseCalc.CMValue;

var
  ValueDlg : PValueDlg;
  I        : longint;

begin
  I := ValueOf(CalcMode,DisplayStr);
  ValueDlg := new(PValueDlg,Init(@Self,MakeIntResource(BCalcValueDlg),I,CalcSize));
  Application^.ExecDialog(ValueDlg)
end;

procedure TBaseCalc.NotImplemented;

begin
  MessageBox(HWindow,'Sorry, that function is not implemented in this version'+
             ' of Base Calculator.','Function not available',
             mb_OK+mb_IconInformation)
end;

procedure TBaseCalc.CMHelpIndex;

begin
  NotImplemented
end;

procedure TBaseCalc.CMHelpKeyboard;

begin
  NotImplemented
end;

procedure TBaseCalc.CMHelpCommands;

begin
  NotImplemented
end;

procedure TBaseCalc.CMHelpProcedures;

begin
  NotImplemented
end;

procedure TBaseCalc.CMUsingHelp;

begin
  WinHelp(HWindow,BaseCalcHelpFile,Help_HelpOnHelp,0)
end;

procedure TBaseCalc.CMAbout;

var
  AboutDlg : PDialog;

begin
  AboutDlg := new(PDialog,Init(@Self,MakeIntResource(BCalcAboutDlg)));
  Application^.ExecDialog(AboutDlg)
end; 

{ Calculator-specific methods }

procedure TBaseCalc.InitCalculator;

begin
  CalcMem := 0;
  Calc_SetInitValues;
  Calc_Clear
end;

procedure TBaseCalc.Calc_SetInitValues;

begin
  case CalcMode of
    Decimal     : begin
                    MaxVal := 9;
                    case CalcSize of
                      ByteSize  : MaxLen := 3;
                      WordSize  : MaxLen := 5;
                      DWordSize : MaxLen := 10
                    end
                  end;   
    Hexadecimal : begin
                    MaxVal := 15;
                    case CalcSize of
                      ByteSize  : MaxLen := 2;
                      WordSize  : MaxLen := 4;
                      DWordSize : MaxLen := 8
                    end
                  end;
    Binary      : begin
                    MaxVal := 1;
                    case CalcSize of
                      ByteSize  : MaxLen := 8;
                      WordSize  : MaxLen := 16;
                      DWordSize : MaxLen := 0     { Not allowed! }
                    end
                  end;
    Octal       : begin
                    MaxVal := 7;
                    case CalcSize of
                      ByteSize  : MaxLen := 3;
                      WordSize  : MaxLen := 6;
                      DWordSize : MaxLen := 11
                    end
                  end
  end;
  case CalcSize of
    ByteSize  : MaxSize := 255;
    WordSize  : MaxSize := 65535;
    DWordSize : MaxSize := MaxLongInt
  end;
  CalcState := cs_First
end;

procedure TBaseCalc.Calc_Clear;

begin
  DisplayStr := '0';
  CalcState := cs_First;
  CalcPrevUsed := false;
  CalcOp := BCalc_EqualsID;
  CalcFirstEquals := true;
  EnableMenuItem(GetMenu(HWindow),cm_Value,mf_ByCommand + mf_Enabled);
  SetMenu(HWindow,GetMenu(HWindow))
end;

procedure TBaseCalc.Calc_NumKey;

var
  C : char;
  St : string;

begin
  if (Value > MaxVal) or (CalcState = cs_Error) then
    begin
      MessageBeep(mb_OK);
      exit
    end;
  if Value <= 9 then
    C := chr(Value + ord('0'))
  else
    C := chr(Value + ord('A') - 10);
  if CalcState = cs_First then
    begin
      DisplayStr := C;
      CalcState := cs_Valid;
      if not CalcFirstEquals then
        begin
          CalcPrevUsed := false;
          CalcOp := BCalc_EqualsID;
          CalcFirstEquals := true
        end
    end
  else
    if (length(DisplayStr) >= MaxLen) or
       (ValueOf(CalcMode,DisplayStr+C) > MaxSize) then
      begin
        MessageBeep(mb_OK);
        exit
      end
    else
      DisplayStr := DisplayStr + C;
  UpdateDisplay
end;

procedure TBaseCalc.Calc_FncKey;

procedure Calc_Back;

begin
  if (CalcState in [cs_Error,cs_First]) then
    begin
      MessageBeep(mb_OK);
      exit
    end;
  if length(DisplayStr) = 1 then
    Calc_Clear
  else
    dec(DisplayStr[0])
end;

procedure Calc_Eval (Key : word);

var
  ErrorOccurred : boolean;

function Eval (Operation : word; Op1, Op2 : longint) : longint;

begin
  ErrorOccurred := false;
  if ((Operation = BCalc_DivID) or (Operation = BCalc_ModID)) and (Op2 = 0) then
    ErrorOccurred := true
  else 
    case Operation of
      BCalc_EqualsID : Eval := Op2;
      BCalc_PlusID   : Eval := Op1 + Op2;
      BCalc_MinusID  : Eval := Op1 - Op2;
      BCalc_TimesID  : Eval := Op1 * Op2;
      BCalc_DivID    : Eval := Op1 div Op2;
      BCalc_ModID    : Eval := Op1 mod Op2;
      BCalc_AndID    : Eval := Op1 and Op2;
      BCalc_OrID     : Eval := Op1 or Op2;
      BCalc_XorID    : Eval := Op1 xor Op2;
      BCalc_EqvID    : Eval := not(Op1 xor Op2)
    end
end;

var
  Op1, Op2, Result : longint;
  St1              : array [0..23] of char;

begin
  Op1 := CalcPrev;
  Op2 := ValueOf(CalcMode,DisplayStr);
  if CalcFirstEquals then
    Result := Eval(CalcOp,Op1,Op2)
  else
    Result := Eval(CalcOp,Op2,Op1);
  if ErrorOccurred then
    begin
      CalcState := cs_Error;
      LoadString(HInstance,str_ErrorID,@St1,sizeof(St1));
      DisplayStr := StrPas(St1);
      EnableMenuItem(GetMenu(HWindow),cm_Value,mf_ByCommand + mf_Grayed + mf_Disabled);
      SetMenu(HWindow,GetMenu(HWindow));
      exit
    end;
  CalcState := cs_First;
  RoundVal(CalcSize,Result);
  DisplayStr := StringOf(CalcMode,Result);
  if Key = BCalc_MPlusID then
    begin
      CalcMem := CalcMem + Result;
      RoundVal(CalcSize,CalcMem);
      CalcOp := BCalc_EqualsID;
      CalcPrevUsed := false;
      CalcFirstEquals := true
    end
  else if CalcFirstEquals then
    begin
      CalcPrev := Op2;
      CalcFirstEquals := false
    end
end;

procedure Calc_DoFunc (Key : word);

begin
  if CalcPrevUsed then
    Calc_Eval(BCalc_EqualsID);
  CalcPrevUsed := true;
  CalcPrev := ValueOf(CalcMode,DisplayStr);
  CalcOp := Key;
  CalcFirstEquals := true
end; 

var
  St, St2 : string;
  I : longint;

begin { Calc_FncKey }
  if CalcState = cs_Error then
    if Key = BCalc_ClrID then
      begin
        Calc_Clear;
        UpdateDisplay
      end
    else
      MessageBeep(mb_OK)
  else
    begin
      case Key of
        BCalc_ClrID    : Calc_Clear;
        BCalc_BackID   : Calc_Back;
        BCalc_NotID    : DisplayStr := StringOf(CalcMode,not ValueOf(CalcMode,DisplayStr));
        BCalc_NegID    : DisplayStr := StringOf(CalcMode,-ValueOf(CalcMode,DisplayStr));
        BCalc_MinID    : CalcMem := ValueOf(CalcMode,DisplayStr);
        BCalc_MRID     : DisplayStr := StringOf(CalcMode,CalcMem);

        BCalc_PlusID,
        BCalc_MinusID,
        BCalc_TimesID,
        BCalc_DivID,
        BCalc_ModID,
        BCalc_AndID,
        BCalc_OrID,
        BCalc_XorID,
        BCalc_EqvID    : Calc_DoFunc(Key);

        BCalc_EqualsID,
        BCalc_MPlusID  : Calc_Eval(Key)
      end;
    if (Key <> BCalc_BacKID) and (Key <> BCalc_EqualsID) and
       (Key <> BCalc_MPlusID) then
      CalcState := cs_First;
    UpdateDisplay
  end
end;

procedure TBaseCalc.Calc_SaveSettings;

var
  St1,
  St2 : string[31];
  I   : word;

begin
  str(X,St1);
  str(Y,St2);
  St1 := '(' + St1 + ',' + St2 + ')'#0;
  LoadString(HInstance,str_OriginID,@St2[1],sizeof(St2)-1);
  WriteProfileString(AppName,@St2[1],@St1[1]);
  case CalcMode of
    Decimal     : I := str_DecID;
    Hexadecimal : I := str_HexID;
    Binary      : I := str_BinID;
    Octal       : I := str_OctID
  end;
  LoadString(HInstance,str_ModeID,@St1[1],sizeof(St1)-1);
  LoadString(HInstance,I,@St2[1],sizeof(St2)-1);
  WriteProfileString(AppName,@St1[1],@St2[1]);
  case CalcSize of
    ByteSize  : I := str_BSizeID;
    WordSize  : I := str_WSizeID;
    DWordSize : I := str_DWSizeID
  end;
  LoadString(HInstance,str_SizeID,@St1[1],sizeof(St1)-1);
  LoadString(HInstance,I,@St2[1],sizeof(St2)-1);
  WriteProfileString(AppName,@St1[1],@St2[1])
end;


{ Base Calculator Application methods }

procedure TBaseCalcApp.InitMainWindow;

begin
  MainWindow := new(PBaseCalc,Init)
end;

procedure TBaseCalcApp.InitInstance;

begin
  TApplication.InitInstance;
  HAccTable := LoadAccelerators(HInstance,MakeIntResource(BCalcKeys))
end;

function TBaseCalcApp.ProcessAppMsg;

begin
  ProcessAppMsg := ProcessAccels(Message) or ProcessDlgMsg(Message)
end;


var
  BaseCalcApp : TBaseCalcApp;

begin { Main }
  BaseCalcApp.Init(AppName);
  BaseCalcApp.Run;
  BaseCalcApp.Done
end.
